#include <3ds.h>
#include <citro3d.h>
#include <citro2d.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CLEAR_TARGET_OPAQUE
#define CLEAR_BEFORE_SCENE

int main(int argc, char** argv)
{
    gfxInitDefault();
    C3D_Init(C3D_DEFAULT_CMDBUF_SIZE);
    C2D_Init(C2D_DEFAULT_MAX_OBJECTS);
    C2D_Prepare();

    consoleInit(GFX_TOP, NULL);
    C3D_RenderTarget* screen = C2D_CreateScreenTarget(GFX_BOTTOM, GFX_LEFT);

    const Tex3DS_SubTexture subtex = {
        512, 256,
        0.0f, 1.0f, 1.0f, 0.0f
    };
    C3D_Tex tex;
    C3D_TexInit(&tex, 512, 256, GPU_RGBA8);
    C3D_RenderTarget* target = C3D_RenderTargetCreateFromTex(&tex, GPU_TEXFACE_2D, 0, -1);
    C2D_Image img = {&tex, &subtex};

    const u32 clear_color_screen = C2D_Color32f(0,0,0,1);
    const u32 gray_color = C2D_Color32f(0.125f,0.125f,0.125f,1);
    #ifdef CLEAR_TARGET_OPAQUE
    const u32 clear_color_target = C2D_Color32f(0,0,0,1);
    #else
    const u32 clear_color_target = C2D_Color32f(0,0,0,0);
    #endif
    
    const u32 color_a = C2D_Color32f(1,0,0,0.5f);
    const u32 color_b = C2D_Color32f(0,1,0,0.5f);
    const u32 color_x = C2D_Color32f(0,0,1,0.5f);
    const u32 color_y = C2D_Color32f(1,1,1,0.5f);

    const u32* color_selected = &color_a;
    touchPosition old_touch;
    bool touching = false;
    bool released = false;

    printf("Offscreen rendertarget test\n");
    printf("Touch the bottom screen to place a colored square centered on the touch point\n");
    printf("Press ABXY to change the color of the square\n");

    printf("\nPress START to quit.\n");

    while(aptMainLoop())
    {
        hidScanInput();
        u32 kDown = hidKeysDown();
        u32 kHeld = hidKeysHeld();

        // Respond to user input
        if(kDown & KEY_START) break;

        if(kDown & KEY_A) color_selected = &color_a;
        else if(kDown & KEY_B) color_selected = &color_b;
        else if(kDown & KEY_X) color_selected = &color_x;
        else if(kDown & KEY_Y) color_selected = &color_y;

        touchPosition touch;
        hidTouchRead(&touch);

        if(kDown & KEY_TOUCH)
        {
            touching = true;
        }
        else if(kHeld & KEY_TOUCH)
        {
            // we're holding
        }
        else if(touching)
        {
            released = true;
            touching = false;
        }

        // Render the scene
        C3D_FrameBegin(C3D_FRAME_SYNCDRAW);

        if(released)
        {
            #ifdef CLEAR_BEFORE_SCENE
            C2D_TargetClear(target, clear_color_target);
            C2D_SceneBegin(target);
            #else
            C2D_SceneBegin(target);
            C2D_TargetClear(target, clear_color_target);
            #endif

            C2D_DrawRectSolid(old_touch.px - 12, old_touch.py - 12, 0.5f, 24, 24, *color_selected);

            released = false;
        }

        #ifdef CLEAR_BEFORE_SCENE
        C2D_TargetClear(screen, clear_color_screen);
        C2D_SceneBegin(screen);
        #else
        C2D_SceneBegin(screen);
        C2D_TargetClear(screen, clear_color_screen);
        #endif

        C2D_DrawRectSolid(32, 32, 0.0f, 256, 176, gray_color);
        C2D_DrawImageAt(img, 0, 0, 0.5f, NULL, 1.0f, 1.0f);

        if(touching)
        {
            C2D_DrawRectSolid(touch.px - 12, touch.py - 12, 1.0f, 24, 24, *color_selected);
        }

        C3D_FrameEnd(0);

        old_touch = touch;
    }

    C3D_RenderTargetDelete(target);
    C3D_TexDelete(&tex);

    C2D_Fini();
    C3D_Fini();
    gfxExit();
    return 0;
}